/**
 * 
 */
package li.qwas.test;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.EntityTransaction;
import javax.persistence.Persistence;

import org.junit.After;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;

/**
 * @author raffael
 */
public class EclipseLinkTest {

	private EntityManager em;

	/**
	 * @throws java.lang.Exception
	 */
	@Before
	public void prepare() throws Exception {
		EntityManagerFactory factory = Persistence.createEntityManagerFactory("test-pu");
		em = factory.createEntityManager();
	}

	/**
	 * @throws java.lang.Exception
	 */
	@After
	public void cleanup() throws Exception {
		em.close();
	}

	@Test
	public void testDirectRemove() {
		EntityTransaction et = em.getTransaction();

		et.begin();
		try {
			B b1 = new B(1);
			em.persist(b1);
			em.flush();
			em.remove(b1);
			em.flush();
			Assert.assertTrue(b1.isPreRemoveCalled());
		} finally {
			et.rollback();
		}
	}

	@Test
	public void testCascateRemove() {
		EntityTransaction et = em.getTransaction();

		et.begin();
		try {
			A a1 = new A(1);
			B b1 = new B(1);
			a1.getBs().add(b1);
			em.persist(a1);
			em.flush();
			em.remove(a1);
			em.flush();
			Assert.assertTrue(b1.isPreRemoveCalled());
		} finally {
			et.rollback();
		}
	}

	@Test
	public void testWithOrphanRemoal() {
		EntityTransaction et = em.getTransaction();

		et.begin();
		try {
			A a1 = new A(1);
			B b1 = new B(1);
			a1.getBs().add(b1);
			em.persist(a1);
			em.flush();
			em.refresh(a1);
			em.refresh(b1);
			a1.getBs().remove(b1);
			em.flush();
			// This fails with eclipseLink but works with openjpa and hibernate
			Assert.assertTrue(b1.isPreRemoveCalled());
		} finally {
			et.rollback();
		}
	}
}
